﻿#region License Information

/* HeuristicLab
 * Copyright (C) 2002-2015 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */

#endregion

using System;
using System.Collections;
using System.Collections.Generic;
using System.Linq;
using HeuristicLab.Common;
using HeuristicLab.Core;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;

namespace HeuristicLab.Problems.DataAnalysis {
  [Item("ModifiableDataset", "Represents a dataset containing data that should be analyzed, which can be modified by adding or replacing variables and values.")]
  [StorableClass]
  public class ModifiableDataset : Dataset {
    [StorableConstructor]
    private ModifiableDataset(bool deserializing) : base(deserializing) { }
    private ModifiableDataset(ModifiableDataset original, Cloner cloner) : base(original, cloner) { }
    public override IDeepCloneable Clone(Cloner cloner) { return new ModifiableDataset(this, cloner); }
    public ModifiableDataset() : base() { }
    public ModifiableDataset(IEnumerable<string> variableNames, IEnumerable<IList> variableValues) : base(variableNames, variableValues) { }

    public void ReplaceRow(int row, IEnumerable<object> values) {
      var list = values.ToList();
      if (list.Count != variableNames.Count)
        throw new ArgumentException("The number of values must be equal to the number of variable names.");
      // check if all the values are of the correct type
      for (int i = 0; i < list.Count; ++i) {
        if (list[i].GetType() != GetVariableType(variableNames[i])) {
          throw new ArgumentException("The type of the provided value does not match the variable type.");
        }
      }
      // replace values
      for (int i = 0; i < list.Count; ++i) {
        variableValues[variableNames[i]][row] = list[i];
      }
    }

    public void AddRow(IEnumerable<object> values) {
      var list = values.ToList();
      if (list.Count != variableNames.Count)
        throw new ArgumentException("The number of values must be equal to the number of variable names.");
      // check if all the values are of the correct type
      for (int i = 0; i < list.Count; ++i) {
        if (list[i].GetType() != GetVariableType(variableNames[i])) {
          throw new ArgumentException("The type of the provided value does not match the variable type.");
        }
      }
      // add values
      for (int i = 0; i < list.Count; ++i) {
        variableValues[variableNames[i]].Add(list[i]);
      }
      rows++;
    }

    // adds a new variable to the dataset
    public void AddVariable<T>(string variableName, IEnumerable<T> values) {
      if (variableValues.ContainsKey(variableName))
        throw new ArgumentException("Variable " + variableName + " is already present in the dataset.");
      int count = values.Count();
      if (count != rows)
        throw new ArgumentException("The number of values must exactly match the number of rows in the dataset.");
      variableValues[variableName] = new List<T>(values);
      variableNames.Add(variableName);
    }

    public void RemoveVariable(string variableName) {
      if (!variableValues.ContainsKey(variableName))
        throw new ArgumentException("The variable " + variableName + " does not exist in the dataset.");
      variableValues.Remove(variableName);
      variableNames.Remove(variableName);
    }

    // slow, avoid to use this
    public void RemoveRow(int row) {
      foreach (var list in variableValues.Values)
        list.RemoveAt(row);
      rows--;
    }

    public void ChangeVariableValue(object value, string variableName, int row) {
      IList list;
      variableValues.TryGetValue(variableName, out list);
      if (list == null)
        throw new ArgumentException("The variable " + variableName + " does not exist in the dataset.");
      if (row < 0 || list.Count < row)
        throw new ArgumentOutOfRangeException("Invalid row value");
      list[row] = value;
    }

    private Type GetVariableType(string variableName) {
      IList list;
      variableValues.TryGetValue(variableName, out list);
      if (list == null)
        throw new ArgumentException("The variable " + variableName + " does not exist in the dataset.");
      return list[0].GetType();
    }
  }
}
